<?php

declare( strict_types=1 );

namespace RankBear\Common\Settings;

use RankBear\Common\Utilities as Utils;

// Abort if this file is called directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( Choices::class ) ) {
	/**
	 * Everything related to setting, getting, and sanitizing plugin settings/options.
	 *
	 * @link https://divpusher.com/blog/wordpress-customizer-sanitization-examples/ Good list of sanitizer functions.
	 */
	class Choices {

		/**
		 * Get an array of options for the supported search engine countries.
		 *
		 * @param bool $for_picking If true, will output for React radio/select input.
		 *
		 * @return array
		 */
		public function get_search_engine_countries(
			bool $for_picking = false
		): array {
			$choices = [
				//'' => _x('Choose…', 'search engine country', 'rankbear' ),
				'AF' => _x( 'Afghanistan', 'search engine country', 'rankbear' ),
				'AX' => _x( 'Åland Islands', 'search engine country', 'rankbear' ),
				'AL' => _x( 'Albania', 'search engine country', 'rankbear' ),
				'DZ' => _x( 'Algeria', 'search engine country', 'rankbear' ),
				'AS' => _x( 'American Samoa', 'search engine country', 'rankbear' ),
				'AD' => _x( 'AndorrA', 'search engine country', 'rankbear' ),
				'AO' => _x( 'Angola', 'search engine country', 'rankbear' ),
				'AI' => _x( 'Anguilla', 'search engine country', 'rankbear' ),
				'AQ' => _x( 'Antarctica', 'search engine country', 'rankbear' ),
				'AG' => _x( 'Antigua and Barbuda', 'search engine country', 'rankbear' ),
				'AR' => _x( 'Argentina', 'search engine country', 'rankbear' ),
				'AM' => _x( 'Armenia', 'search engine country', 'rankbear' ),
				'AW' => _x( 'Aruba', 'search engine country', 'rankbear' ),
				'AU' => _x( 'Australia', 'search engine country', 'rankbear' ),
				'AT' => _x( 'Austria', 'search engine country', 'rankbear' ),
				'AZ' => _x( 'Azerbaijan', 'search engine country', 'rankbear' ),
				'BS' => _x( 'Bahamas', 'search engine country', 'rankbear' ),
				'BH' => _x( 'Bahrain', 'search engine country', 'rankbear' ),
				'BD' => _x( 'Bangladesh', 'search engine country', 'rankbear' ),
				'BB' => _x( 'Barbados', 'search engine country', 'rankbear' ),
				'BY' => _x('Belarus', 'search engine country', 'rankbear' ),
				'BE' => _x('Belgium', 'search engine country', 'rankbear' ),
				'BZ' => _x('Belize', 'search engine country', 'rankbear' ),
				'BJ' => _x('Benin', 'search engine country', 'rankbear' ),
				'BM' => _x('Bermuda', 'search engine country', 'rankbear' ),
				'BT' => _x('Bhutan', 'search engine country', 'rankbear' ),
				'BO' => _x('Bolivia', 'search engine country', 'rankbear' ),
				'BA' => _x('Bosnia and Herzegovina', 'search engine country', 'rankbear' ),
				'BW' => _x('Botswana', 'search engine country', 'rankbear' ),
				'BV' => _x('Bouvet Island', 'search engine country', 'rankbear' ),
				'BR' => _x('Brazil', 'search engine country', 'rankbear' ),
				'IO' => _x('British Indian Ocean Territory', 'search engine country', 'rankbear' ),
				'BN' => _x('Brunei Darussalam', 'search engine country', 'rankbear' ),
				'BG' => _x('Bulgaria', 'search engine country', 'rankbear' ),
				'BF' => _x('Burkina Faso', 'search engine country', 'rankbear' ),
				'BI' => _x('Burundi', 'search engine country', 'rankbear' ),
				'KH' => _x('Cambodia', 'search engine country', 'rankbear' ),
				'CM' => _x('Cameroon', 'search engine country', 'rankbear' ),
				'CA' => _x('Canada', 'search engine country', 'rankbear' ),
				'CV' => _x('Cape Verde', 'search engine country', 'rankbear' ),
				'KY' => _x('Cayman Islands', 'search engine country', 'rankbear' ),
				'CF' => _x('Central African Republic', 'search engine country', 'rankbear' ),
				'TD' => _x('Chad', 'search engine country', 'rankbear' ),
				'CL' => _x('Chile', 'search engine country', 'rankbear' ),
				'CN' => _x('China', 'search engine country', 'rankbear' ),
				'CX' => _x('Christmas Island', 'search engine country', 'rankbear' ),
				'CC' => _x('Cocos (Keeling) Islands', 'search engine country', 'rankbear' ),
				'CO' => _x('Colombia', 'search engine country', 'rankbear' ),
				'KM' => _x('Comoros', 'search engine country', 'rankbear' ),
				'CG' => _x('Congo', 'search engine country', 'rankbear' ),
				'CD' => _x('Congo, The Democratic Republic of the', 'search engine country', 'rankbear' ),
				'CK' => _x('Cook Islands', 'search engine country', 'rankbear' ),
				'CR' => _x('Costa Rica', 'search engine country', 'rankbear' ),
				'CI' => _x('Cote DIvoire', 'search engine country', 'rankbear' ),
				'HR' => _x('Croatia', 'search engine country', 'rankbear' ),
				'CU' => _x('Cuba', 'search engine country', 'rankbear' ),
				'CY' => _x('Cyprus', 'search engine country', 'rankbear' ),
				'CZ' => _x('Czech Republic', 'search engine country', 'rankbear' ),
				'DK' => _x('Denmark', 'search engine country', 'rankbear' ),
				'DJ' => _x('Djibouti', 'search engine country', 'rankbear' ),
				'DM' => _x('Dominica', 'search engine country', 'rankbear' ),
				'DO' => _x('Dominican Republic', 'search engine country', 'rankbear' ),
				'EC' => _x('Ecuador', 'search engine country', 'rankbear' ),
				'EG' => _x('Egypt', 'search engine country', 'rankbear' ),
				'SV' => _x('El Salvador', 'search engine country', 'rankbear' ),
				'GQ' => _x('Equatorial Guinea', 'search engine country', 'rankbear' ),
				'ER' => _x('Eritrea', 'search engine country', 'rankbear' ),
				'EE' => _x('Estonia', 'search engine country', 'rankbear' ),
				'ET' => _x('Ethiopia', 'search engine country', 'rankbear' ),
				'FK' => _x('Falkland Islands (Malvinas)', 'search engine country', 'rankbear' ),
				'FO' => _x('Faroe Islands', 'search engine country', 'rankbear' ),
				'FJ' => _x('Fiji', 'search engine country', 'rankbear' ),
				'FI' => _x('Finland', 'search engine country', 'rankbear' ),
				'FR' => _x('France', 'search engine country', 'rankbear' ),
				'GF' => _x('French Guiana', 'search engine country', 'rankbear' ),
				'PF' => _x('French Polynesia', 'search engine country', 'rankbear' ),
				'TF' => _x('French Southern Territories', 'search engine country', 'rankbear' ),
				'GA' => _x('Gabon', 'search engine country', 'rankbear' ),
				'GM' => _x('Gambia', 'search engine country', 'rankbear' ),
				'GE' => _x('Georgia', 'search engine country', 'rankbear' ),
				'DE' => _x('Germany', 'search engine country', 'rankbear' ),
				'GH' => _x('Ghana', 'search engine country', 'rankbear' ),
				'GI' => _x('Gibraltar', 'search engine country', 'rankbear' ),
				'GR' => _x('Greece', 'search engine country', 'rankbear' ),
				'GL' => _x('Greenland', 'search engine country', 'rankbear' ),
				'GD' => _x('Grenada', 'search engine country', 'rankbear' ),
				'GP' => _x('Guadeloupe', 'search engine country', 'rankbear' ),
				'GU' => _x('Guam', 'search engine country', 'rankbear' ),
				'GT' => _x('Guatemala', 'search engine country', 'rankbear' ),
				'GG' => _x('Guernsey', 'search engine country', 'rankbear' ),
				'GN' => _x('Guinea', 'search engine country', 'rankbear' ),
				'GW' => _x('Guinea-Bissau', 'search engine country', 'rankbear' ),
				'GY' => _x('Guyana', 'search engine country', 'rankbear' ),
				'HT' => _x('Haiti', 'search engine country', 'rankbear' ),
				'HM' => _x('Heard Island and Mcdonald Islands', 'search engine country', 'rankbear' ),
				'VA' => _x('Holy See (Vatican City State)', 'search engine country', 'rankbear' ),
				'HN' => _x('Honduras', 'search engine country', 'rankbear' ),
				'HK' => _x('Hong Kong', 'search engine country', 'rankbear' ),
				'HU' => _x('Hungary', 'search engine country', 'rankbear' ),
				'IS' => _x('Iceland', 'search engine country', 'rankbear' ),
				'IN' => _x('India', 'search engine country', 'rankbear' ),
				'ID' => _x('Indonesia', 'search engine country', 'rankbear' ),
				'IR' => _x('Iran, Islamic Republic Of', 'search engine country', 'rankbear' ),
				'IQ' => _x('Iraq', 'search engine country', 'rankbear' ),
				'IE' => _x('Ireland', 'search engine country', 'rankbear' ),
				'IM' => _x('Isle of Man', 'search engine country', 'rankbear' ),
				'IL' => _x('Israel', 'search engine country', 'rankbear' ),
				'IT' => _x('Italy', 'search engine country', 'rankbear' ),
				'JM' => _x('Jamaica', 'search engine country', 'rankbear' ),
				'JP' => _x('Japan', 'search engine country', 'rankbear' ),
				'JE' => _x('Jersey', 'search engine country', 'rankbear' ),
				'JO' => _x('Jordan', 'search engine country', 'rankbear' ),
				'KZ' => _x('Kazakhstan', 'search engine country', 'rankbear' ),
				'KE' => _x('Kenya', 'search engine country', 'rankbear' ),
				'KI' => _x('Kiribati', 'search engine country', 'rankbear' ),
				'KP' => _x('Korea, Democratic PeopleS Republic of', 'search engine country', 'rankbear' ),
				'KR' => _x('Korea, Republic of', 'search engine country', 'rankbear' ),
				'KW' => _x('Kuwait', 'search engine country', 'rankbear' ),
				'KG' => _x('Kyrgyzstan', 'search engine country', 'rankbear' ),
				'LA' => _x('Lao PeopleS Democratic Republic', 'search engine country', 'rankbear' ),
				'LV' => _x('Latvia', 'search engine country', 'rankbear' ),
				'LB' => _x('Lebanon', 'search engine country', 'rankbear' ),
				'LS' => _x('Lesotho', 'search engine country', 'rankbear' ),
				'LR' => _x('Liberia', 'search engine country', 'rankbear' ),
				'LY' => _x('Libyan Arab Jamahiriya', 'search engine country', 'rankbear' ),
				'LI' => _x('Liechtenstein', 'search engine country', 'rankbear' ),
				'LT' => _x('Lithuania', 'search engine country', 'rankbear' ),
				'LU' => _x('Luxembourg', 'search engine country', 'rankbear' ),
				'MO' => _x('Macao', 'search engine country', 'rankbear' ),
				'MK' => _x('Macedonia, The Former Yugoslav Republic of', 'search engine country', 'rankbear' ),
				'MG' => _x('Madagascar', 'search engine country', 'rankbear' ),
				'MW' => _x('Malawi', 'search engine country', 'rankbear' ),
				'MY' => _x('Malaysia', 'search engine country', 'rankbear' ),
				'MV' => _x('Maldives', 'search engine country', 'rankbear' ),
				'ML' => _x('Mali', 'search engine country', 'rankbear' ),
				'MT' => _x('Malta', 'search engine country', 'rankbear' ),
				'MH' => _x('Marshall Islands', 'search engine country', 'rankbear' ),
				'MQ' => _x('Martinique', 'search engine country', 'rankbear' ),
				'MR' => _x('Mauritania', 'search engine country', 'rankbear' ),
				'MU' => _x('Mauritius', 'search engine country', 'rankbear' ),
				'YT' => _x('Mayotte', 'search engine country', 'rankbear' ),
				'MX' => _x('Mexico', 'search engine country', 'rankbear' ),
				'FM' => _x('Micronesia, Federated States of', 'search engine country', 'rankbear' ),
				'MD' => _x('Moldova, Republic of', 'search engine country', 'rankbear' ),
				'MC' => _x('Monaco', 'search engine country', 'rankbear' ),
				'MN' => _x('Mongolia', 'search engine country', 'rankbear' ),
				'MS' => _x('Montserrat', 'search engine country', 'rankbear' ),
				'MA' => _x('Morocco', 'search engine country', 'rankbear' ),
				'MZ' => _x('Mozambique', 'search engine country', 'rankbear' ),
				'MM' => _x('Myanmar', 'search engine country', 'rankbear' ),
				'NA' => _x('Namibia', 'search engine country', 'rankbear' ),
				'NR' => _x('Nauru', 'search engine country', 'rankbear' ),
				'NP' => _x('Nepal', 'search engine country', 'rankbear' ),
				'NL' => _x('Netherlands', 'search engine country', 'rankbear' ),
				'AN' => _x('Netherlands Antilles', 'search engine country', 'rankbear' ),
				'NC' => _x('New Caledonia', 'search engine country', 'rankbear' ),
				'NZ' => _x('New Zealand', 'search engine country', 'rankbear' ),
				'NI' => _x('Nicaragua', 'search engine country', 'rankbear' ),
				'NE' => _x('Niger', 'search engine country', 'rankbear' ),
				'NG' => _x('Nigeria', 'search engine country', 'rankbear' ),
				'NU' => _x('Niue', 'search engine country', 'rankbear' ),
				'NF' => _x('Norfolk Island', 'search engine country', 'rankbear' ),
				'MP' => _x('Northern Mariana Islands', 'search engine country', 'rankbear' ),
				'NO' => _x('Norway', 'search engine country', 'rankbear' ),
				'OM' => _x('Oman', 'search engine country', 'rankbear' ),
				'PK' => _x('Pakistan', 'search engine country', 'rankbear' ),
				'PW' => _x('Palau', 'search engine country', 'rankbear' ),
				'PS' => _x('Palestinian Territory, Occupied', 'search engine country', 'rankbear' ),
				'PA' => _x('Panama', 'search engine country', 'rankbear' ),
				'PG' => _x('Papua New Guinea', 'search engine country', 'rankbear' ),
				'PY' => _x('Paraguay', 'search engine country', 'rankbear' ),
				'PE' => _x('Peru', 'search engine country', 'rankbear' ),
				'PH' => _x('Philippines', 'search engine country', 'rankbear' ),
				'PN' => _x('Pitcairn', 'search engine country', 'rankbear' ),
				'PL' => _x('Poland', 'search engine country', 'rankbear' ),
				'PT' => _x('Portugal', 'search engine country', 'rankbear' ),
				'PR' => _x('Puerto Rico', 'search engine country', 'rankbear' ),
				'QA' => _x('Qatar', 'search engine country', 'rankbear' ),
				'RE' => _x('Reunion', 'search engine country', 'rankbear' ),
				'RO' => _x('Romania', 'search engine country', 'rankbear' ),
				'RU' => _x('Russian Federation', 'search engine country', 'rankbear' ),
				'RW' => _x('RWANDA', 'search engine country', 'rankbear' ),
				'SH' => _x('Saint Helena', 'search engine country', 'rankbear' ),
				'KN' => _x('Saint Kitts and Nevis', 'search engine country', 'rankbear' ),
				'LC' => _x('Saint Lucia', 'search engine country', 'rankbear' ),
				'PM' => _x('Saint Pierre and Miquelon', 'search engine country', 'rankbear' ),
				'VC' => _x('Saint Vincent and the Grenadines', 'search engine country', 'rankbear' ),
				'WS' => _x('Samoa', 'search engine country', 'rankbear' ),
				'SM' => _x('San Marino', 'search engine country', 'rankbear' ),
				'ST' => _x('Sao Tome and Principe', 'search engine country', 'rankbear' ),
				'SA' => _x('Saudi Arabia', 'search engine country', 'rankbear' ),
				'SN' => _x('Senegal', 'search engine country', 'rankbear' ),
				'CS' => _x('Serbia and Montenegro', 'search engine country', 'rankbear' ),
				'SC' => _x('Seychelles', 'search engine country', 'rankbear' ),
				'SL' => _x('Sierra Leone', 'search engine country', 'rankbear' ),
				'SG' => _x('Singapore', 'search engine country', 'rankbear' ),
				'SK' => _x('Slovakia', 'search engine country', 'rankbear' ),
				'SI' => _x('Slovenia', 'search engine country', 'rankbear' ),
				'SB' => _x('Solomon Islands', 'search engine country', 'rankbear' ),
				'SO' => _x('Somalia', 'search engine country', 'rankbear' ),
				'ZA' => _x('South Africa', 'search engine country', 'rankbear' ),
				'GS' => _x('South Georgia and the South Sandwich Islands', 'search engine country', 'rankbear' ),
				'ES' => _x('Spain', 'search engine country', 'rankbear' ),
				'LK' => _x('Sri Lanka', 'search engine country', 'rankbear' ),
				'SD' => _x('Sudan', 'search engine country', 'rankbear' ),
				'SR' => _x('Suriname', 'search engine country', 'rankbear' ),
				'SJ' => _x('Svalbard and Jan Mayen', 'search engine country', 'rankbear' ),
				'SZ' => _x('Swaziland', 'search engine country', 'rankbear' ),
				'SE' => _x('Sweden', 'search engine country', 'rankbear' ),
				'CH' => _x( 'Switzerland', 'search engine country', 'rankbear' ),
				'SY' => _x( 'Syrian Arab Republic', 'search engine country', 'rankbear' ),
				'TW' => _x( 'Taiwan, Province of China', 'search engine country', 'rankbear' ),
				'TJ' => _x( 'Tajikistan', 'search engine country', 'rankbear' ),
				'TZ' => _x( 'Tanzania, United Republic of', 'search engine country', 'rankbear' ),
				'TH' => _x( 'Thailand', 'search engine country', 'rankbear' ),
				'TL' => _x( 'Timor-Leste', 'search engine country', 'rankbear' ),
				'TG' => _x( 'Togo', 'search engine country', 'rankbear' ),
				'TK' => _x( 'Tokelau', 'search engine country', 'rankbear' ),
				'TO' => _x( 'Tonga', 'search engine country', 'rankbear' ),
				'TT' => _x( 'Trinidad and Tobago', 'search engine country', 'rankbear' ),
				'TN' => _x( 'Tunisia', 'search engine country', 'rankbear' ),
				'TR' => _x( 'Turkey', 'search engine country', 'rankbear' ),
				'TM' => _x( 'Turkmenistan', 'search engine country', 'rankbear' ),
				'TC' => _x( 'Turks and Caicos Islands', 'search engine country', 'rankbear' ),
				'TV' => _x( 'Tuvalu', 'search engine country', 'rankbear' ),
				'UG' => _x( 'Uganda', 'search engine country', 'rankbear' ),
				'UA' => _x( 'Ukraine', 'search engine country', 'rankbear' ),
				'AE' => _x( 'United Arab Emirates', 'search engine country', 'rankbear' ),
				'GB' => _x( 'United Kingdom (UK|GB)', 'search engine country', 'rankbear' ),
				'US' => _x( 'United States (USA)', 'search engine country', 'rankbear' ),
				'UM' => _x( 'United States Minor Outlying Islands', 'search engine country', 'rankbear' ),
				'UY' => _x( 'Uruguay', 'search engine country', 'rankbear' ),
				'UZ' => _x( 'Uzbekistan', 'search engine country', 'rankbear' ),
				'VU' => _x( 'Vanuatu', 'search engine country', 'rankbear' ),
				'VE' => _x( 'Venezuela', 'search engine country', 'rankbear' ),
				'VN' => _x( 'Viet Nam', 'search engine country', 'rankbear' ),
				'VG' => _x( 'Virgin Islands, British', 'search engine country', 'rankbear' ),
				'VI' => _x( 'Virgin Islands, U.S.', 'search engine country', 'rankbear' ),
				'WF' => _x( 'Wallis and Futuna', 'search engine country', 'rankbear' ),
				'EH' => _x( 'Western Sahara', 'search engine country', 'rankbear' ),
				'YE' => _x( 'Yemen', 'search engine country', 'rankbear' ),
				'ZM' => _x( 'Zambia', 'search engine country', 'rankbear' ),
				'ZW' => _x( 'Zimbabwe', 'search engine country', 'rankbear' ),
			];

			$result = [];

			// Allow to unset value, and our <Select> allows a Placeholder.
			if ( empty( $for_picking ) ) {
				$result[''] = '';
			}

			foreach ( $choices as $key => $value ) {
				if ( $for_picking ) {
					$result[] = [
						'label' => $value,
						'value' => $key,
						//'isDisabled' => ( 'US' === $key ) ? true : false, // let's always disable United States
					];
				} else {
					$result[ $key ] = $value;
				}
			}

			return $result;
		}

		/**
		 * Get the Post Types options.
		 *
		 * @param string $format Without a value, will work with Customizer. Pass 'RadioControl', for example, to format
		 *                       the output for a <RadioControl> React component.
		 *
		 * @return array
		 */
		public function get_choices_post_types( $format = '' ): array {
			$result = [];

			foreach ( ( new Utils\Posts() )->get_public_post_types() as $type ) {
				if ( 'RadioControl' === $format ) {
					$result[] = [
						'label' => $type->label,
						'value' => $type->name,
					];
				} else {
					// name is the registered name and label is what the user sees.
					$result[ $type->name ] = $type->label;
				}
			}

			return $result;
		}

	}
}
